package Slovo::Command;
use Mojo::Base 'Mojolicious::Command';

1;

=encoding utf8

=head1 NAME

Slovo::Command - Slovo Command base class

=head1 DESCRIPTION

L<Slovo::Command> is an abstract base class for L<Slovo> commands. For now we
just use it as namespace. Slovo::Command ISA L<Mojolicious::Command>.


=head1 SEE ALSO

See L<Mojolicious::Commands/"COMMANDS"> for a list of commands that are
available by default in Mojolicious.

L<Mojolicious::Command>

=cut

