@charset "utf-8";

label.field-with-error { color: #dd7e5e }
input.field-with-error { background-color: #fd9e7e }

/**
 * Body CSS
 */
html,
body {
  height: 100%;
  background-color: #eee;
  font-size:18px;
  /* TODO: change body/html font-size dynamically to change the lenght values
   * to everything set in "rem"s (root ems). Useful for screens with bigger
   * resolution where the font is too small.
   */
}

html,
body,
input,
textarea,
button {
  -webkit-font-smoothing: antialiased;
  -moz-osx-font-smoothing: grayscale;
  text-shadow: 1px 1px 1px rgba(0, 0, 0, 0.004);
}

/**
 * Header and footer CSS
 */
header, footer {
  position: fixed;
  right: 0;
  left: 0;
  transition: left 0.2s;
}

header {
  top: 0;
}

footer {
    bottom: 0;
    padding: 0 1rem 0 1rem;
}

header a, footer a {
  color: white;
}

.mui--appbar-min-height, .mui-appbar, .mui--appbar-height {
    min-height: 1rem;
    max-height: 6rem;
    cursor: default;
}
.mui--appbar-line-height {
    line-height:1.5rem;
}
footer.mui-appbar table,
header.mui-appbar table {
    height: 100%;
    width:  100%;
    /* top | right | bottom | left */
    padding: 0 1rem 0 1rem;
}

header.mui-appbar .mui--text-right a,
header.mui-appbar .mui--text-title a,
header.mui-appbar .mui--text-title,
span.title {
    font-size: 0.9rem;
}

#logo {
    display: inline;
    width: 1rem;
    height: 1rem;
    vertical-align: middle;
}
/* disable user selection */
header, footer, #sidedrawer {
    -webkit-touch-callout: none;
    -webkit-user-select: none;
    -khtml-user-select: none;
    -moz-user-select: none;
    -ms-user-select: none;
    user-select: none;
}

#sidedrawer {
  position: fixed;
  top: 0;
  bottom: 0;
  width: 200px;
  left: -200px;
  overflow: auto;
  z-index: 2;
  background-color: #fff;
  transition: transform 0.2s;
}

.sidedrawer-toggle {
  color: #fff;
  cursor: pointer;
  font-size: 1rem;
  line-height: 1.2rem;
  margin-right: 0.45rem;
}

.sidedrawer-toggle:hover {
  color: #fff;
  text-decoration: none;
}



/**
 * Content CSS
 */
main#content-wrapper {
  min-height: 100%;
  overflow-x: hidden;
  margin-left: 0px;
  transition: margin-left 0.2s;

  /* sticky bottom */
  margin-bottom: -3rem;
  padding: 1.5rem 1rem 0 1rem;
}

@media (min-width: 60rem) {
  section {
      margin:auto;
      max-width:55rem;
  }
}

footer {
  margin-left: 0px;
  transition: margin-left 0.2s;
}

@media (min-width: 768px) {
  header {
    left: 200px;
  }

  #sidedrawer {
    transform: translate(200px);
  }

  main#content-wrapper {
    margin-left: 200px;
  }

  footer {
    margin-left: 200px;
  }

  body.hide-sidedrawer header {
    left: 0;
  }

  body.hide-sidedrawer #sidedrawer {
    transform: translate(0px);
  }

  body.hide-sidedrawer main#content-wrapper {
    margin-left: 0;
  }

  body.hide-sidedrawer footer {
    margin-left: 0;
  }
}


/**
 * Toggle Side drawer
 */
#sidedrawer.active {
  transform: translate(200px);
}

/**
 * Side drawer CSS
 */
#sidedrawer-brand {
  padding-left: 1.1rem;
}

#sidedrawer ul {
  list-style: none;
  margin: 0;
}

#sidedrawer ul {
  padding-left: 0;
}

#sidedrawer > ul > li:first-child {
  padding-top: .1rem;
}

#sidedrawer ul li strong, #sidedrawer ul li div {
  display: block;
  padding: .2rem .6rem;
  cursor: pointer;
}
/* second level */
#sidedrawer ul li > ul li strong,
#sidedrawer ul li > ul li div {
  padding-left: 1rem;
}
/* third level */
#sidedrawer ul li > ul li > ul li strong,
#sidedrawer ul li > ul li > ul li div {
  padding-left: 1.8rem;
}
/* fourth level */
#sidedrawer ul li > ul li > ul li > ul li strong,
#sidedrawer ul li > ul li > ul li > ul li div {
  padding-left: 2.6rem;
}

#sidedrawer ul li strong:hover,
#sidedrawer ul li div:hover {
  background-color: #E0E0E0;
}


#manage {
    font-size: x-small
}

main section {
    padding: 1rem;
}

/* Re-set some rules for this site. */
header .mui-dropdown .mui-btn--small,
header .mui-dropdown .mui-dropdown__menu > li > a {
    height: 1rem;
    line-height: 1rem;
    padding: 0 0.5rem;
    font-size: 0.9rem;
    margin: 0.3rem 0.3rem 0.3rem 0.3rem !important;
}

.mui-dropdown__menu {
    min-width: initial;
}

/* home page */
section.group {

}

section.group article.writing h2 {
    height: 2rem;
    overflow: hidden;
    /* top | right | bottom | left */
    margin: 0 0      .5rem   .5rem;
    padding: 0 1.5rem 0      0;
    border-top: 0.5rem solid #607d8b;
    border-right: 0.5rem solid #607d8b;
}

section.group article.writing a.writing {
    /* top | right | bottom | left */
    padding: 0 .5rem     0    0;
    border-right: 0.5rem solid #607d8b;
    border-bottom: 0.5rem solid #607d8b;
    position: absolute;
    top: 1.5rem;
    right: 0;
}

section.group article.writing {
    display: inline-block;
    max-width: 100%;
    min-width: 15rem;
    width: 25rem;
    margin: auto;
    position: relative;
    background: white;
}

section.group article.writing div.exerpt {
    overflow: hidden;
    height: 8rem;
    padding: .5rem;
}

section p.drop-cap::first-letter {
  float: left;
  font-size: 5rem;
  line-height: .68;
  font-weight: bold;
  /* top | right | bottom | left */
  margin: .05em .1em 0 0;
  text-transform: uppercase;
  font-family: BukyvedeRegular;
}
