@charset "utf-8";

html,
body {
  height: 100%;
  background-color: #eee;
  font-size:16px;
  /* TODO: change body/html font-size dynamically to change the lenght values
   * to everything set in "rem"s (root ems). Useful for screens with bigger
   * resolution where the font is too small.
   */
}

html,
body,
input,
textarea,
button {
  -webkit-font-smoothing: antialiased;
  -moz-osx-font-smoothing: grayscale;
  text-shadow: 1px 1px 1px rgba(0, 0, 0, 0.004);
}

.mui-textfield > .field-with-error,
.field-with-error {
    color: #fff;
    background-color:  #F44336;
    font-size:1.5em;
}

.hidden, .deleted, .disabled, .expired, .upcoming {opacity: 0.50;}
.deleted, .disabled {text-decoration: line-through red;}
.upcoming {text-decoration: green wavy overline;}
.expired {text-decoration: green wavy underline;}
#logo {
    height: 1rem;
    vertical-align:middle;
}

.item-field {
    border:  1px #ddd solid;
    margin:  1ex 0 1ex 1ex;
    display: block;
}

td.buttons {
    font-size: 1.5rem;
}

input, textarea, option, button {
    font-size: 1rem;
}

.mui--appbar-min-height, .mui-appbar, .mui--appbar-height {
    min-height: 1rem;
    max-height: 6rem;
    cursor: default;
}

footer.mui-appbar table, header.mui-appbar table {
    height: 100%;
    width: 100%;
    padding: 0 1rem 0 1rem;
}

header a, footer a {
    color: white;
}

.b {
    font-weight:bold;
}

.mui-select > label,
.mui-textfield > label {
    line-height: 1.1em;
    font-size: 1.2rem;
}

.trumbowyg-editor {
    font-family: Veleka, FreeSerif, BukyvedeRegular, serif;
}

.mui-panel {
    padding: .3rem;
}

.mui-panel.breadcrumb a:first-child {
    float:right;
}
/* Pages: Font Awesome 5 Free styles */
.pages .mui-dropdown {
    /* float:right; */
}

.pages .mui-dropdown__menu li a {
    font-family: "Font Awesome 5 Free";
}
.pages .mui-btn--small {
    width: 1.2rem;
    height: 1.2rem;
    line-height: 1.2rem;
    padding: .1rem;
}

.mui-btn--small .mui-caret {margin:0;}


.pages ul .fa-li {
    left: -1em;
    width: 100%;
    /* border-top: 1px dotted; */
    text-align: left;
}

.mui-form>legend {font-size: 2rem;}
.mui-form>fieldset>legend {
    font-size: 1.5rem;
    cursor: pointer;
}

.mui-form>fieldset>legend {color: rgba(0,0,0,.54);}
.mui-form>fieldset>legend:hover {color: rgba(0,0,0,1);}

.mui-select__menu {
    z-index:100;
}
.mui-select__menu>.mui--is-selected {
    font-weight: bold;
}
